@@ -208,12 +208,29 @@ stack_pop(GList **stack)
 	g_free(s);
 }
 
+static void
+set_display (GnomeCalc *gc)
+{
+	GtkWidget *label = gc->_priv->text_display;
+	gchar *string;
+
+	string = g_strconcat ("<span foreground=\"white\"  font_desc=\"Helvetica 24\">", gc->_priv->result_string, "</span>", NULL);
+	gtk_label_set_markup (GTK_LABEL (label), string);
+	g_free (string);
+}
+
 static void
 do_error(GnomeCalc *gc)
 {
 	gc->_priv->error = TRUE;
-	strcpy(gc->_priv->result_string,"e");
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
+	if (errno == ERANGE)
+		strcpy(gc->_priv->result_string,_("inf"));
+	else
+		strcpy(gc->_priv->result_string,"e");
+
+	set_display (gc);
+
+
 }
 
 /*we handle sigfpe's so that we can find all the errors*/
@@ -281,8 +298,10 @@ reduce_stack(GnomeCalc *gc)
 
 	if(errno>0 ||
 	   finite(stack->d.number)==0) {
-		errno = 0;
 		do_error(gc);
+		stack->d.number = 0;
+		errno = 0;
+		gc->_priv->error = FALSE;
 	}
 }
 
@@ -383,7 +402,7 @@ set_result(GnomeCalc *gc)
 		return;
 
 	gc->_priv->result = stack->d.number;
-
+	
         /* make sure put values in a consistent manner */
 	/* XXX: perhaps we can make sure the calculator works on all locales,
 	 * but currently it will lose precision if we don't do this */
@@ -398,8 +417,7 @@ set_result(GnomeCalc *gc)
 	strncpy(gc->_priv->result_string,buf,12);
 	gc->_priv->result_string[12]='\0';
 
-	gtk_entry_set_text (GTK_ENTRY(gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
 	gtk_signal_emit(GTK_OBJECT(gc),
 			gnome_calc_signals[RESULT_CHANGED_SIGNAL],
 			gc->_priv->result);
@@ -442,15 +460,30 @@ convert_num(gdouble num, GnomeCalcMode from, GnomeCalcMode to)
 {
 	if(to==from)
 		return num;
-	else if(from==GNOME_CALC_DEG)
-		if(to==GNOME_CALC_RAD)
+	else if(from==GNOME_CALC_DEG) {
+		if(to==GNOME_CALC_RAD) {
+			if (fabs (num - 90) < 0.000001)
+				return M_PI_2;
+			else if (fabs (num - 180) < 0.000001)
+				return M_PI;
+			else if (fabs (num - 45) < 0.000001)
+				return M_PI_4;
+			else if (fabs (num - 360) < 0.000001)
+				return 2*M_PI;
 			return (num*M_PI)/180;
-		else /*GRAD*/
+		} else /*GRAD*/
 			return (num*200)/180;
+	}
 	else if(from==GNOME_CALC_RAD)
-		if(to==GNOME_CALC_DEG)
+		if(to==GNOME_CALC_DEG) {
+			if (num == M_PI)
+				return 180;
+			else if (num == M_PI_2)
+				return 90;
+			else if (num == M_PI_4)
+				return 45;
 			return (num*180)/M_PI;
-		else /*GRAD*/
+		} else /*GRAD*/
 			return (num*200)/M_PI;
 	else /*GRAD*/
 		if(to==GNOME_CALC_DEG)
@@ -511,13 +544,13 @@ simple_func(GtkWidget *w, gpointer data)
 		unselect_invert(gc);
 		return;
 	}
-
+	
 	/*only convert non inverting functions*/
 	if(!gc->_priv->invert && but->convert_to_rad)
 		stack->d.number = convert_num(stack->d.number,
 					      gc->_priv->mode,
 					      GNOME_CALC_RAD);
-
+	
 	errno = 0;
 	{
 		sighandler_t old = signal(SIGFPE,sigfpe_handler);
@@ -527,20 +560,22 @@ simple_func(GtkWidget *w, gpointer data)
 			stack->d.number = (*invfunc)(stack->d.number);
 		signal(SIGFPE,old);
 	}
-
+	
 	if(errno>0 ||
 	   finite(stack->d.number)==0) {
-		errno = 0;
 		do_error(gc);
+		gc->_priv->error = FALSE;
+		stack->d.number = 0;
+		errno = 0;
 		return;
 	}
-
+	
 	/*we are converting back from rad to mode*/
 	if(gc->_priv->invert && but->convert_to_rad)
 		stack->d.number = convert_num(stack->d.number,
 					      GNOME_CALC_RAD,
 					      gc->_priv->mode);
-
+	
 	set_result(gc);
 
 	unselect_invert(gc);
@@ -718,8 +753,8 @@ add_digit (GtkWidget *w, gpointer data)
 
 	strcat (gc->_priv->result_string, digit);
 
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
+ 	
         /* make sure get values in a consistent manner */
 	gnome_i18n_push_c_numeric_locale ();
 	sscanf(gc->_priv->result_string, "%lf", &gc->_priv->result);
@@ -758,7 +793,8 @@ negate_val(GtkWidget *w, gpointer data)
 		else
 			*p='-';
 	} else {
-		if(gc->_priv->result_string[0]=='-')
+		if(gc->_priv->result_string[0]=='-' || 
+		   gc->_priv->result == 0)
 			gc->_priv->result_string[0]=' ';
 		else
 			gc->_priv->result_string[0]='-';
@@ -768,8 +804,8 @@ negate_val(GtkWidget *w, gpointer data)
 	gnome_i18n_pop_c_numeric_locale ();
 	sscanf(gc->_priv->result_string, "%lf", &gc->_priv->result);
 	gnome_i18n_push_c_numeric_locale ();
-	
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
+
+	set_display (gc);
 }
 
 static gdouble
@@ -861,7 +897,7 @@ static void
 store_m(GtkWidget *w, gpointer data)
 {
 	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
-
+	
 	g_return_if_fail(gc!=NULL);
 
 	if(gc->_priv->error)
@@ -871,8 +907,8 @@ store_m(GtkWidget *w, gpointer data)
 
 	gc->_priv->memory = gc->_priv->result;
 
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
+ 	
 	unselect_invert(gc);
 }
 
@@ -895,7 +931,7 @@ static void
 sum_m(GtkWidget *w, gpointer data)
 {
 	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
-
+	
 	g_return_if_fail(gc!=NULL);
 
 	if(gc->_priv->error)
@@ -905,8 +941,8 @@ sum_m(GtkWidget *w, gpointer data)
 
 	gc->_priv->memory += gc->_priv->result;
 
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
+ 	
 	unselect_invert(gc);
 }
 
@@ -1061,6 +1097,44 @@ sub_parenth(GtkWidget *w, gpointer data)
 	unselect_invert(gc);
 }
 
+static double
+sin_helper (double value)
+{
+	double ret = sin (value);
+	
+	if (fabs (ret) <= fabs (sin (M_PI))) 
+		return 0.0;
+	else
+		return ret;
+}
+
+static double
+cos_helper (double value)
+{
+	double ret = cos (value);
+	if (fabs (ret) <= fabs (cos (M_PI_2)))
+		return 0.0;
+	else
+		return ret;
+}
+
+static double
+tan_helper (double value)
+{
+	double ret = tan (value);
+	double temp1, temp2;
+
+	if (fabs (ret) >= fabs (tan(3 * M_PI_2))) {
+		errno=ERANGE;
+		return 0.0;
+	}
+	else if (fabs (ret) <= fabs (tan(M_PI))) {
+		return 0.0;
+	}
+	else
+		return ret;
+}
+
 static const CalculatorButton buttons[8][5] = {
 	{
 		{N_("1/x"),  (GtkSignalFunc)simple_func, c_inv,  NULL,   FALSE, {0}, N_("Inverse"), NULL },
@@ -1070,9 +1144,9 @@ static const CalculatorButton buttons[8][5] = {
 		{N_("AC"),   (GtkSignalFunc)reset_calc,  NULL,   NULL,   FALSE, {'a','A', GDK_Escape, 0}, N_("Reset"), NULL }
 	},{
 		{N_("INV"),  NULL,                       NULL,   NULL,   FALSE, {'i','I',0}, N_("Shift"), NULL }, /*inverse button*/
-		{N_("sin"),  (GtkSignalFunc)simple_func, sin,    asin,   TRUE,  {'s','S',0}, N_("Sine"), NULL },
-		{N_("cos"),  (GtkSignalFunc)simple_func, cos,    acos,   TRUE,  {'c','C',0}, N_("Cosine"), NULL },
-		{N_("tan"),  (GtkSignalFunc)simple_func, tan,    atan,   TRUE,  {'t','T',0}, N_("Tangent"), NULL },
+		{N_("sin"),  (GtkSignalFunc)simple_func, sin_helper, asin, TRUE,{'s','S',0}, N_("Sine"), NULL },
+		{N_("cos"),  (GtkSignalFunc)simple_func, cos_helper, acos, TRUE,{'c','C',0}, N_("Cosine"), NULL },
+		{N_("tan"),  (GtkSignalFunc)simple_func, tan_helper,    atan,   TRUE,  {'t','T',0}, N_("Tangent"), NULL },
 		{N_("DEG"),  (GtkSignalFunc)drg_toggle,  NULL,   NULL,   FALSE, {'d','D',0}, N_("Switch degrees / radians / grad"), NULL }
 	},{
 		{N_("e"),    (GtkSignalFunc)set_e,       NULL,   NULL,   FALSE, {'e','E',0}, N_("Base of Natural Logarithm"), NULL },
@@ -1134,7 +1208,7 @@ create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 				   but->signal_func,
 				   (gpointer) but);
 	}
-
+	
 	for(i=0;but->keys[i]!=0;i++) {
 		gtk_widget_add_accelerator(w, "clicked",
 					   gc->_priv->accel,
@@ -1172,26 +1246,30 @@ create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 static void
 gnome_calc_instance_init (GnomeCalc *gc)
 {
+	GtkWidget *hbox, *event;
 	gint x,y;
 	GtkWidget *table;
-	PangoFontDescription *pdesc;
-
+	GdkColor color;
+	
 	gc->_priv = g_new0(GnomeCalcPrivate, 1);
-
-	gc->_priv->text_display = gtk_entry_new ();
-	gtk_editable_set_editable (GTK_EDITABLE (gc->_priv->text_display), FALSE);
-
-	gtk_box_pack_start (GTK_BOX (gc), gc->_priv->text_display, FALSE, FALSE, 0);
-	gtk_widget_set_direction (gc->_priv->text_display, GTK_TEXT_DIR_RTL);
-
-	pdesc = pango_font_description_from_string ("Helvetica 24");
-	if (pdesc != NULL) {
-		gtk_widget_modify_font (gc->_priv->text_display, pdesc);
-		pango_font_description_free (pdesc);
-	}
-
+	
+	event = gtk_event_box_new ();
+	gtk_widget_show (event);
+	gdk_color_parse ("#000000", &color);
+	gtk_widget_modify_bg (event, GTK_STATE_NORMAL, &color); 
+	gtk_box_pack_start (GTK_BOX (gc), event, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (event), 1);
+	
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (event), hbox);
+	
+	gc->_priv->text_display = gtk_label_new (NULL);
+	gtk_label_set_selectable (GTK_LABEL (gc->_priv->text_display), TRUE);
+	/*gtk_label_set_justify (GTK_LABEL (gc->_priv->text_display), GTK_JUSTIFY_RIGHT);*/
 	gtk_widget_show (gc->_priv->text_display);
-
+	gtk_box_pack_end (GTK_BOX (hbox), gc->_priv->text_display, FALSE, TRUE, 0);	
+	
 	gc->_priv->stack = NULL;
 	gc->_priv->result = 0;
 	strcpy(gc->_priv->result_string," 0");
@@ -1205,15 +1283,18 @@ gnome_calc_instance_init (GnomeCalc *gc)
 	table = gtk_table_new(8,5,TRUE);
 	gtk_widget_show(table);
 
-	gtk_box_pack_end(GTK_BOX(gc),table,TRUE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(gc),table,TRUE,TRUE,0);
 
 	for(x=0;x<5;x++) {
 		for(y=0;y<8;y++) {
 			create_button(gc, table, x, y);
 		}
-	}
+	}	
+	
 	gtk_tooltips_enable (gc->_priv->tooltips);
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
+
+        set_display (gc);
+	
 }
 
 
@@ -1380,9 +1461,9 @@ backspace_cb (GtkWidget *w, gpointer data)
 		if (stack->type == CALCULATOR_NUMBER)
 			stack->d.number = gc->_priv->result;
 	}
-	
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
 
+	set_display (gc);
+ 	
 	/* make sure get values in a consistent manner */
 	old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
