@@ -40,7 +40,7 @@ create_button (const gchar *label, GtkSignalFunc func)
     button = gtk_button_new_with_label (label);
     GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
     if (func != NULL)
-        gtk_signal_connect (GTK_OBJECT (button), "clicked",
+        g_signal_connect (G_OBJECT (button), "clicked",
           func, NULL);
     gtk_widget_show (button);
     return button;
@@ -74,13 +74,20 @@ create_chartable (void)
               (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
               0, 0);
 
-            gtk_signal_connect (GTK_OBJECT (button), "clicked",
-              GTK_SIGNAL_FUNC (cb_charbtn_click), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "enter",
+            g_signal_connect (G_OBJECT (button), "clicked",
+              G_CALLBACK (cb_charbtn_click), NULL);
+            g_signal_connect (G_OBJECT (button), "enter",
+              G_CALLBACK (cb_charbtn_enter), NULL);
+            g_signal_connect (G_OBJECT (button), "leave",
+              G_CALLBACK (cb_charbtn_leave), NULL);
+
+
+            g_signal_connect (G_OBJECT (button), "focus_in_event",
               GTK_SIGNAL_FUNC (cb_charbtn_enter), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "leave",
+            g_signal_connect (G_OBJECT (button), "focus_out_event",
               GTK_SIGNAL_FUNC (cb_charbtn_leave), NULL);
 
+
         }
     }
 
@@ -107,13 +114,20 @@ create_chartable (void)
               (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
               0, 0);
 
-            gtk_signal_connect (GTK_OBJECT (button), "clicked",
-              GTK_SIGNAL_FUNC (cb_charbtn_click), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "enter",
+            g_signal_connect (G_OBJECT (button), "clicked",
+              G_CALLBACK (cb_charbtn_click), NULL);
+            g_signal_connect (G_OBJECT (button), "enter",
+              G_CALLBACK (cb_charbtn_enter), NULL);
+            g_signal_connect (G_OBJECT (button), "leave",
+              G_CALLBACK (cb_charbtn_leave), NULL);
+
+
+            g_signal_connect (G_OBJECT (button), "focus_in_event",
               GTK_SIGNAL_FUNC (cb_charbtn_enter), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "leave",
+            g_signal_connect (G_OBJECT (button), "focus_out_event",
               GTK_SIGNAL_FUNC (cb_charbtn_leave), NULL);
 
+
         }
     }
 
@@ -121,6 +135,45 @@ create_chartable (void)
     return chartable;
 }
 
+/* Check if gail is loaded */
+gboolean
+check_gail(GtkWidget *widget)
+{
+   return GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(widget));
+}
+
+
+/* Add AtkName and AtkDescription */
+void
+add_atk_namedesc(GtkWidget *widget, const gchar *name, const gchar *desc)
+{
+   AtkObject *atk_widget;
+   atk_widget = gtk_widget_get_accessible(widget);
+   atk_object_set_name(atk_widget, name);
+   atk_object_set_description(atk_widget, desc);
+}
+
+
+/* Add AtkRelation */
+void
+add_atk_relation(GtkWidget *obj1, GtkWidget *obj2, AtkRelationType type)
+{
+
+    AtkObject *atk_obj1, *atk_obj2;
+    AtkRelationSet *relation_set;
+    AtkRelation *relation;
+
+    atk_obj1 = gtk_widget_get_accessible(obj1);
+
+    atk_obj2 = gtk_widget_get_accessible(obj2);
+
+    relation_set = atk_object_ref_relation_set (atk_obj1);
+    relation = atk_relation_new(&atk_obj2, 1, type);
+    atk_relation_set_add(relation_set, relation);
+    g_object_unref(G_OBJECT (relation));
+
+}
+
 
 static void
 main_app_create_ui (MainApp *app)
@@ -132,6 +185,7 @@ main_app_create_ui (MainApp *app)
     GtkWidget *buttonbox, *button;
     GtkWidget *viewport;
 
+
     /* Main window */
     {
         BonoboDockLayoutItem *item;
@@ -139,8 +193,8 @@ main_app_create_ui (MainApp *app)
         app->window = gnome_app_new (_(PACKAGE), _("Gnome Character Map"));
         gtk_widget_set_name (app->window, "mainapp");
         
-        gtk_signal_connect_object (GTK_OBJECT (app->window), "destroy",
-          GTK_SIGNAL_FUNC (main_app_destroy), G_OBJECT (app));
+        g_signal_connect_swapped (G_OBJECT (app->window), "destroy",
+          G_CALLBACK (main_app_destroy), G_OBJECT (app));
         gtk_widget_realize (app->window);
 	
         appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
@@ -180,25 +234,33 @@ main_app_create_ui (MainApp *app)
           "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-*-*");
         gtk_button_set_relief (GTK_BUTTON (app->fontpicker), GTK_RELIEF_NONE);
         gtk_box_pack_start (GTK_BOX (hbox), app->fontpicker, FALSE, TRUE, 0);
-        gtk_signal_connect (GTK_OBJECT (app->fontpicker), "font_set",
-          GTK_SIGNAL_FUNC (cb_fontpicker_font_set), NULL);
+        g_signal_connect (G_OBJECT (app->fontpicker), "font_set",
+          G_CALLBACK (cb_fontpicker_font_set), NULL);
 
         vsep = gtk_vseparator_new ();
         gtk_box_pack_start (GTK_BOX (hbox), vsep, FALSE, FALSE, 0);
 #endif	
         alabel = gtk_label_new_with_mnemonic (_("_Text to copy:"));
+        gtk_misc_set_padding (GTK_MISC (alabel), GNOME_PAD_SMALL, -1);
         gtk_box_pack_start (GTK_BOX (hbox), alabel, FALSE, TRUE, 0);
 
         app->entry = gtk_entry_new ();
         gtk_label_set_mnemonic_widget (GTK_LABEL (alabel), app->entry);
         gtk_box_pack_start (GTK_BOX (hbox), app->entry, TRUE, TRUE, 0);
-        
+
         button = gtk_button_new_with_mnemonic (_("_Copy"));
         gtk_container_set_border_width (GTK_CONTAINER (button), 2);
         g_signal_connect (G_OBJECT (button), "clicked",
         		  G_CALLBACK (cb_copy_click), NULL);
-        gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);		  
-
+        gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);		 
+        if (check_gail(app->entry))
+        {
+          add_atk_namedesc(GTK_WIDGET(alabel), _("Text to copy"), _("Text to copy"));
+          add_atk_namedesc(app->entry, _("Entry"), _("Text to copy"));
+          add_atk_namedesc(button, _("Copy"), _("Copy the text"));
+          add_atk_relation(app->entry, GTK_WIDGET(alabel), ATK_RELATION_LABELLED_BY);
+        }
+ 
         gtk_widget_show_all (hbox);
         item = g_list_nth_data (GNOME_APP (app->window)->layout->items, 0);
         app->textbar = GTK_WIDGET (item->item);
@@ -220,8 +282,8 @@ main_app_create_ui (MainApp *app)
         font = gdk_fontset_load (
           _("-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-*-*,*-r-*")
         );
-	if (font != NULL)
-		gtk_style_set_font(app->btnstyle, font);
+        if (font != NULL)
+               gtk_style_set_font(app->btnstyle, font);
 #endif
         gtk_widget_destroy (tmp);
 
@@ -270,8 +332,8 @@ main_app_create_ui (MainApp *app)
         font = gdk_fontset_load (
           _("-*-helvetica-bold-r-normal-*-*-180-*-*-p-*-*-*,*-r-*")
         );
-	if (font != NULL)
-		gtk_style_set_font(style, font);
+        if (font != NULL)
+               gtk_style_set_font(style, font);
 
         gtk_widget_set_style (viewport, style);
         gtk_box_pack_start (GTK_BOX (vbox2), viewport, FALSE, TRUE, 0);
@@ -281,14 +343,20 @@ main_app_create_ui (MainApp *app)
         gtk_widget_set_style (app->preview_label, style);
     }
 #endif
+    /* some default prefs */
+    app->insert_at_end = FALSE;
+    
     gtk_widget_show_all (vbox);
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
       view_menu[0].widget), TRUE);
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
       view_menu[1].widget), TRUE);
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
+      settings_menu[0].widget), app->insert_at_end);
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
       settings_menu[1].widget), TRUE);
     gtk_widget_grab_focus (app->entry);
+    
 }
 
 
