@@ -1,4 +1,4 @@
-/* $Id: gdict-app.c,v 1.32 2002/01/09 02:02:32 kevinv Exp $ */
+/* $Id: gdict-app.c,v 1.36 2002/03/21 03:33:47 kevinv Exp $ */
 /* -*- mode: c; style: k&r; c-basic-offset: 4 -*- */
 
 /*
@@ -49,6 +49,7 @@ GDictDefbox *defbox;
 GDictSpeller *speller;
 GtkWidget *pref_dialog;
 GtkWidget *socket_error_dialog;
+gboolean  gail_loaded = FALSE;
 
 #ifdef HAVE_GNOME_PRINT
 GnomePrinter *gdict_printer = NULL;
@@ -161,10 +162,10 @@ gdict_open_speller (void)
                             GTK_SIGNAL_FUNC (socket_error_cb), NULL);
         gtk_signal_connect (GTK_OBJECT (speller), "close",
                             GTK_SIGNAL_FUNC (spell_close_cb), NULL);
+        gtk_widget_show_all (GTK_WIDGET (speller));
     }
-    
-    if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (speller)))
-    	gtk_widget_show (GTK_WIDGET (speller));
+    else
+    	gtk_window_present (GTK_WINDOW (speller));
 }
 
 
@@ -464,11 +465,8 @@ void
 gdict_app_show_preferences (void) 
 {
 	
-    if (!pref_dialog) {
-    g_print ("show prefs \n");
+    if (!pref_dialog)
         pref_dialog = gdict_pref_dialog_new (context);
-        
-    }
 
     gtk_widget_show (pref_dialog);
 }
@@ -485,6 +483,63 @@ about_cb (GtkMenuItem *menuitem, gpointer user_data)
     gdict_about();
 }
 
+
+/*
+ * add_atk_namedesc
+ * @widget    : The Gtk Widget for which @name and @desc are added.
+ * @name      : Accessible Name
+ * @desc      : Accessible Description
+ * Description: This function adds accessible name and description to a
+ *              Gtk widget.
+ */
+
+void
+add_atk_namedesc(GtkWidget *widget, const gchar *name, const gchar *desc)
+{
+    AtkObject *atk_widget;
+
+    g_return_if_fail (GTK_IS_WIDGET(widget));
+
+    atk_widget = gtk_widget_get_accessible(widget);
+    
+    if( name != NULL )
+    atk_object_set_name(atk_widget, name);
+    if( desc !=NULL )
+    atk_object_set_description(atk_widget, desc);
+}
+
+
+/*
+ * add_atk_relation
+ * @obj1      : The first widget in the relation @rel_type
+ * @obj2      : The second widget in the relation @rel_type.
+ * @rel_type  : Relation type which relates @obj1 and @obj2
+ * Description: This function establishes Atk Relation between two given
+ *              objects.
+ */
+
+void
+add_atk_relation(GtkWidget *obj1, GtkWidget *obj2, AtkRelationType rel_type)
+{
+
+    AtkObject *atk_obj1, *atk_obj2;
+    AtkRelationSet *relation_set;
+    AtkRelation *relation;
+
+    g_return_if_fail (GTK_IS_WIDGET(obj1));
+    g_return_if_fail (GTK_IS_WIDGET(obj2));
+
+    atk_obj1 = gtk_widget_get_accessible(obj1);
+
+    atk_obj2 = gtk_widget_get_accessible(obj2);
+
+    relation_set = atk_object_ref_relation_set (atk_obj1);
+    relation = atk_relation_new(&atk_obj2, 1, rel_type);
+    atk_relation_set_add(relation_set, relation);
+    g_object_unref(G_OBJECT (relation));
+
+}
+
 static GnomeUIInfo file_menu_uiinfo[] = {
     GNOMEUIINFO_ITEM_STOCK (N_("Lookup"),
 			    N_("Lookup word in dictionary"),
@@ -542,30 +597,6 @@ static GnomeUIInfo menubar_uiinfo[] = {
     GNOMEUIINFO_END
 };
 
-static GnomeUIInfo toolbar_uiinfo[] = {
-    GNOMEUIINFO_ITEM_STOCK (N_("Lookup"), 
-			    N_("Lookup word in dictionary"), 
-			    lookup_cb, GNOME_STOCK_PIXMAP_SEARCH),
-    GNOMEUIINFO_ITEM_STOCK (N_("Spell"), 
-			    N_("Check word spelling"), 
-			    spell_cb, GNOME_STOCK_PIXMAP_SPELLCHECK),
-    GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK (N_("Cut"), 
-			    N_("Cut to Clipboard"), 
-			    cut_cb, GNOME_STOCK_PIXMAP_CUT),
-    GNOMEUIINFO_ITEM_STOCK (N_("Copy"), 
-			    N_("Copy to Clipboard"), 
-			    copy_cb, GNOME_STOCK_PIXMAP_COPY),
-    GNOMEUIINFO_ITEM_STOCK (N_("Paste"), 
-			    N_("Paste Clipboard Contents"), 
-			    paste_cb, GNOME_STOCK_PIXMAP_PASTE),
-    GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK (N_("Settings"), 
-			  N_("Server to use, default search strategy, fonts"), 
-			    preferences_cb, GNOME_STOCK_PIXMAP_PREFERENCES),
-    GNOMEUIINFO_END
-};
-
 enum
 {
   TARGET_STRING,
@@ -580,7 +611,7 @@ GtkWidget *gdict_app_create (gboolean applet) {
     GtkWidget *vbox;
     GtkWidget *hbox;
     GtkWidget *button;
-    GtkWidget *label;
+    GtkWidget *label, *word_label;
     GtkWidget *scrolled;
     GtkWidget *defbox_vscrollbar;
     GtkTooltips *tooltips;
@@ -594,7 +625,7 @@ GtkWidget *gdict_app_create (gboolean applet) {
   
     tooltips = gtk_tooltips_new ();
   
-    gdict_app = gnome_app_new ("Dictionary", "Dictionary");
+    gdict_app = gnome_app_new ("gnome-dictionary", "Dictionary");
     gtk_window_set_default_size (GTK_WINDOW (gdict_app), 450, 330);
     
     dock = GNOME_APP (gdict_app)->dock;
@@ -605,7 +636,6 @@ GtkWidget *gdict_app_create (gboolean applet) {
 		&applet_close_item, sizeof(GnomeUIInfo));
 
     gnome_app_create_menus (GNOME_APP (gdict_app), menubar_uiinfo);
-    gnome_app_create_toolbar (GNOME_APP (gdict_app), toolbar_uiinfo);
     
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
@@ -616,14 +646,18 @@ GtkWidget *gdict_app_create (gboolean applet) {
     gtk_widget_show (hbox);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
     
-    label = gtk_label_new_with_mnemonic (_("_Lookup"));
+    word_label = gtk_label_new_with_mnemonic (_("_Word : "));
+    gtk_widget_show ( GTK_WIDGET(word_label) );
+    gtk_box_pack_start (GTK_BOX (hbox), word_label, FALSE, FALSE, 0);
+    
+    label = gtk_label_new_with_mnemonic (_("_Look Up"));
     gtk_widget_show (label);
     gtk_misc_set_padding (GTK_MISC (label), 2, 1);
     
     button = gtk_button_new ();
     gtk_container_add (GTK_CONTAINER (button), label);
     gtk_widget_show (button);
-    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
     gtk_drag_dest_set (button, GTK_DEST_DEFAULT_ALL, drop_targets, 
     		       G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
     g_signal_connect (G_OBJECT (button), "clicked",
@@ -633,13 +667,23 @@ GtkWidget *gdict_app_create (gboolean applet) {
 
     gnome_word_entry = gnome_entry_new ("Wordentry");
     gtk_widget_show (gnome_word_entry);
-    /*gtk_box_pack_start (GTK_BOX (vbox), gnome_word_entry, FALSE, FALSE, 0);*/
     gtk_box_pack_start (GTK_BOX (hbox), gnome_word_entry, TRUE, TRUE, 0);
-    GTK_WIDGET_SET_FLAGS (gnome_word_entry, GTK_CAN_FOCUS);
-    gtk_widget_grab_focus (gnome_word_entry);
     gnome_entry_set_max_saved (GNOME_ENTRY (gnome_word_entry), 50);
     word_entry = gnome_entry_gtk_entry(GNOME_ENTRY (gnome_word_entry));
-        
+   
+    gtk_label_set_mnemonic_widget (GTK_LABEL (word_label), word_entry);
+    GTK_WIDGET_SET_FLAGS (word_entry, GTK_CAN_FOCUS);
+    gtk_widget_grab_focus (word_entry);
+
+    if ( GTK_IS_ACCESSIBLE (gtk_widget_get_accessible(gnome_word_entry)))
+    {
+        gail_loaded = TRUE;
+        add_atk_namedesc(GTK_WIDGET(word_label), _("Word"), _("Word"));
+        add_atk_namedesc(gnome_word_entry, _("Word Entry"), _("Enter a Word or select one from the list below"));
+        add_atk_relation(gnome_word_entry, GTK_WIDGET(word_label), ATK_RELATION_LABELLED_BY);
+        add_atk_namedesc(GTK_WIDGET(button), NULL, _("Look Up for a Word"));
+    }
+
     scrolled = gtk_scrolled_window_new (NULL, NULL);
     gtk_container_set_border_width (GTK_CONTAINER (scrolled), GNOME_PAD_SMALL);    
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
